#!/usr/bin/perl
#
# This script shamelessly includes the entire code for logtail
# - which is part of the logcheck package. logtail is GNU Copyleft'ed
# - so is this. 
# 
# logtail authors are:
#    
# Copyright (C) 2003 Jonathan Middleton <jjm@ixtab.org.uk>
# Copyright (C) 2001 Paul Slootman <paul@debian.org>
#
# This merger with Qmail-Scanner code by:
#
# Copyright (C) 2003 Jason Haar <jhaar@users.sourceforge.net>
#
#



use Getopt::Long;
use POSIX;

my ($counter,$is_local,$is_remote,$is_spam,$quarantined,$is_rbl)=0;

my ($logfile)='/var/log/messages';
my ($offsetfile)="$ENV{'HOME'}/qs2mrtg.offset";
my ($mrtg_dir)="/var/www/html/mrtg/";

$uptime=`uptime`;
chomp($uptime);
$uptime=~s/^\s+//;
#$uptime=~s/(^\s+|\s+$)//g;
$output="0\n0\n0\n0\n0\n0\n$uptime\n$hostname\n";

$hn=`hostname`;
$hn=~s/\s//g;


#getopts('f:h');
$result=GetOptions ('h' => \$opt_h, 'help' => \$opt_h,'syslog-file:s' => \$logfile, 'offset-file:s' => \$offsetfile,'mailsrv-name:s' => \$hostname, 'mrtg-output-dir:s' => \$mrtg_dir);

if (! -r $logfile  || $opt_h || $result eq "") {
  print "

Format:

logfile=$logfile
offset-file=$offsetfile

$0 --syslog-file /path/to/syslog/file [ --offset-file /path/to/offsetfile ] \
   [--mrtg-output-dir=/path/to/web/mrtg/] [--mailsrv-name mail.server.name]

This script monitors the syslog file containing Qmail-Scanner logs.

It parses the output and generates mrtg stats

Defaults to: syslog-file=$logfile
             offset-file=$offsetfile
             mrtg-output-dir=$mrtg_dir
             mailsrv-name=$hn


";
  exit 1;
}

if (!$hostname) {
  $hostname=$hn;
}








if (! -d "$mrtg_dir") {
  $DD=`mkdir -p $mrtg_dir`;
}
chdir($mrtg_dir)|| die "cannot chdir to $mrtg_dir - $!";

&logtail;
exit;

sub get_stats {
  my($line)=@_;
  if ($line =~ / tcpserver: [0-9]+\.[0-9]+ rblsmtpd: | qmail-scanner\[[0-9]+\]: (SA:SPAM-QUARANTINED|GL:greylisted)/) {
	$is_rbl++;
  }elsif ($line =~ / qmail-scanner\[[0-9]+\]: ([^\s]+)\s([0-9\.]+)\s([0-9]+)\s([^\s]+)\s([^\s]+)\s([^\s]+)\s([^\s]+)\s(.*)$/) {
    chomp($line);
#    print "line=$line\n";
    $counter++;
    $stats=$1;
    $process_time=$2;
    $size=$3;
    $sender=$4;
    $recip=$5;
    $subj=$6;
    $msgid=$7;
    $the_rest=$8;
   #Get SA data
    if ($stats =~ /:SA:1/) {
      $is_spam++;
    }
    if ($stats =~ /RC[:\(]1/) {
      $is_local++;
    } else {
      $is_remote++;
    }
    #print "$stats: $subj\n";
    if ($sender =~ /^(\<\>)$/) {
      $is_bounce++;
    }
    if ($stats =~ /^(Clear|SA:SPAM-QUARANTINED)/) {
      $not_quarantined++;
      #print "quar=$not_quarantined\n";
    } else {
      #print "nasty: $line\n";
      $quarantined++;
      $stats =~ /^([^:]+:[^:]+)/;
      if ($1 =~ /^Policy/) {
	$policy++;
      } else {
        $virus++;
      }
    }
    $is_rbl=0 if (!$is_rbl);
    $is_spam=0 if (!$is_spam);
    $quarantined=0 if (!$quarantined);
    $counter=0 if (!$counter);
    $is_bounce=0 if (!$is_bounce);
    $is_local=0 if (!$is_local);
    $is_remote=0 if (!$is_remote);
    $virus=0 if (!$virus);
    $policy=0 if (!$policy);
    open(Q,">$mrtg_dir/quarantined.tmp")||die "cannot open for write $mrtg_dir/quarantined.tmp - $!";
    print Q "$virus\n$policy\n$uptime\n$hostname\n";
    close (Q)|| die "cannot close quarantined.tmp - $!";
    rename ("$mrtg_dir/quarantined.tmp","$mrtg_dir/quarantined.txt");
    open(S,">$mrtg_dir/spam.tmp")||die "cannot open for write $mrtg_dir/spam.tmp - $!";
    print S "$is_spam\n$is_rbl\n$uptime\n$hostname\n";
    close (S)|| die "cannot close spam.tmp - $!";
    rename ("$mrtg_dir/spam.tmp","$mrtg_dir/spam.txt");
    open(A,">$mrtg_dir/total.tmp")||die "cannot open for write $mrtg_dir/total.tmp - $!";
    print A "$is_local\n$is_remote\n$uptime\n$hostname\n";
    close (A)|| die "cannot close total.tmp - $!";
    rename ("$mrtg_dir/total.tmp","$mrtg_dir/total.txt");
    #print "out=$output";
  }
  return $output;
}




sub logtail {
unless (open(LOGFILE, $logfile)) {
    print "File $logfile cannot be read.\n";
    exit 66;
}

my ($inode, $offset) = (0, 0);

if (open(OFFSET, $offsetfile)) {
    $_ = <OFFSET>;
    unless (! defined $_) {
	chomp $_;
	$inode = $_;
	$_ = <OFFSET>;
	unless (! defined $_) {
	    chomp $_;
	    $offset = $_;
	}
    }
}

my ($ino, $size);
unless ((undef,$ino,undef,undef,undef,undef,undef,$size) = stat $logfile) {
    print "Cannot get $logfile file size.\n", $logfile;
    exit 65;
}

if ($inode == $ino) {
    exit 0 if $offset == $size; # short cut
    if ($offset > $size) {
        $offset = 0;
        print "***************\n";
        print "*** WARNING ***: Log file $logfile is smaller than last time checked!\n";
        print "*************** This could indicate tampering.\n";
    }
}
if ($inode != $ino || $offset > $size) {
    $offset = 0;
}

seek(LOGFILE, $offset, 0);

while (<LOGFILE>) {
  &get_stats($_);
}

$size = tell LOGFILE;
close LOGFILE;

#print $output;

unless (open(OFFSET, ">$offsetfile")) {
    print "File $offsetfile cannot be created. Check your permissions.\n";
    exit 73;
}
unless (chmod 0600, $offsetfile) {
    print "Cannot set permissions on file $offsetfile\n";
    exit 65;
}
print OFFSET "$ino\n$size\n";
close OFFSET;

exit 0;
}
