#!/usr/bin/perl
#
# Copyright (C) 2005 Jason Haar <jhaar@users.sourceforge.net>

# This script should be called from cron on an hourly basis 
# (even more often for high load sites). It will move the 
# high-scored SPAM caught by Qmail-Scanner into Courier-IMAP oriented
# mail subdirs under the home directory of qscand. This will allow 
# you to connect via IMAP and see all the high-scoring SPAM separated out 
# - where each email recipient gets their own subdir. This won't 
# magically allow your users to log in via IMAP and read just their email 
# - this is a solution for Admins. Someone else can figure out how to do 
# per-user ;-)


#This script must be run as root as a cronjob - typically every hour to ensure
# any job run doesn't have too many files to deal with
#
#


use POSIX;
use MIME::Base64;
use File::Copy;

#Qmail-Scanner dir where high-scoring SPAM is kept
my $quarantine_qscan='/var/spool/qscan/quarantine/spam';


# mail moved to the IMAP maildir that is older than this 
# in days is deleted

$MAX_AGE=14;

# This is the IMAP account IS people can log into to
# view blocked SPAM
$mailbox_owner='qspam';


if ($< ne "0") {
        print "

FATAL ERROR:

This must be run as root as it reads mail from Qmail-Scanner dirs,
and writes them into $mailbox_owner maildirs

";
        exit 1;
}

$check_user=`id -u -n $mailbox_owner 2>/dev/null`;
chomp($check_user);

if ($mailbox_owner ne $check_user) {
  die "User $mailbox_owner does not appear to exist. Exiting...";
}
 

#Get homedir for $mailbox_owner user
my ($name,$passwd,$uid,$gid,$quota,$comment,$gcos,$dir,$shell,$expire) = getpwnam($mailbox_owner);

#Can now set this to the top-level IMAP maildir where to move this SPAM to (assumes Courier-IMAP is being used)
my $imap_maildir="$dir/Maildir";


umask(0077);

if (!-d $imap_maildir) {
  system("maildirmake $imap_maildir");
  system("chown -R $uid:$gid $imap_maildir");
}

chdir ("$quarantine_qscan/new")||die "cannot find $quarantine_qscan/new - $!";

opendir(DIR,".");

my @all_spam_msgs = grep(!/^\.+$/, readdir(DIR));
closedir(DIR);
foreach $spam (@all_spam_msgs) {
  my ($found_sender,$found_recip,$single_recipient,$mfrom,$rto,$b64rto,$subject)='';
  @QSCAN=`tail -6 $quarantine_qscan/new/$spam 2>/dev/null`;	
  #print "QSCAN=@QSCAN\n";
  foreach $line (@QSCAN) {
    if ($line=~/^X-Qmail-Scanner-Mail-From: \"(.*)\" via/) {
      $found_sender=1;
      $mfrom=$1;
    }
    
    #print "line=$line\n";
    if ($line=~/^X-Qmail-Scanner-Rcpt-To: \"(.*)\"$/) {
      $found_recip=1;
      $rto=tolower($1);
      $rto=~s/^\s+//g;
      $b64rto=encode_base64($rto);
      $b64rto=~s/(\r|\n)//g;
    }
  }
  if ($found_sender == 1 && $found_recip == 1) {
    #print "Found $spam to \"$rto\"\n";
    $rto=$rto.",";
    foreach $single_recipient (split(/\,/, $rto)) {
      #print "unaltered \"$single_recipient\"\n";
      $single_recipient=~s/^\s//g;
      $single_recipient=~s/\s$//g;
      $single_recipient=~s/,$//;
      #Replace dots as they are folder separators under Courier-IMAP
      $single_recipient=~s/\./,/g;
      $single_recipient=~s/[^a-z,0-9\-\_\+\=\%\@\~\^\:]/_/ig;
      #print "-- Found $spam to \"$single_recipient\"\n";
      if ($single_recipient ne "") {
	#I'm assuming you're using a Courier-IMAP server - so all
	#subfolders begin with a "dot"
	$single_recipient=".".$single_recipient;
	
	
	if (! -d "$imap_maildir/$single_recipient/new") {
	  #print "maildirmake $imap_maildir/$single_recipient\n";
	  system("setuidgid $mailbox_owner maildirmake $imap_maildir/$single_recipient");
	  if (! -d "$imap_maildir/$single_recipient/new") {
	    die "failed to run setuidgid $mailbox_owner maildirmake $imap_maildir/$single_recipient - $!";
	  }
	}
	#print "transfer($quarantine_qscan/new/$spam,$imap_maildir/$single_recipient)\n";
	&transfer("$quarantine_qscan/new/$spam","$imap_maildir/$single_recipient");
      }else{
	#print "skipped - no recip\n";
      }
    }
    unlink("$quarantine_qscan/new/$spam");
  } else {
    die "Whoa! \"$spam\" is not an official quarantine message (mfrom=$mfrom,rto=$rto).\n";
  }
}

#Finally, expire spam older than $MAX_AGE
&expire_old_spam;

exit;

#####################
##
#####################


sub transfer {
  my ($old,$new)=@_;
  if (-f "$old") {
   my ($filen);
   ($filen=$old)=~s/^.*\///g;
   #print "copy $old $new/tmp\n";
   open(OLD,"<$old")|| die "Something went wrong - cannot read $old - $!";
   open(TMP,">$new/tmp/$filen")|| die "Something went wrong - cannot open for write $new/tmp/$filen - $!";
   while (<OLD>) {
    if (!/^\*\*\* Qmail-Scanner Quarantine Envelope Details Begin \*\*\*/) {
      print TMP;
    }else{
      last;
    }
   }
   close(TMP)|| die "Something went wrong - cannot close $new/tmp/$filen - $!";
   close OLD;
   chown $uid, $gid, glob("$new/tmp/$filen");
   rename("$new/tmp/$filen","$new/new/$filen") || die "Something went wrong - cannot rename $new/tmp/$filen to $new/new/$filen - $!";
   #Finally, delete original
   #unlink("$quarantine_qscan/new/$old");
  }
}

sub expire_old_spam {
  chdir($imap_maildir);
  system("setuidgid $mailbox_owner find $imap_maildir -type f -mtime +${MAX_AGE} -exec rm -f \{\} \\;");
}
